c*********************************************************
c
c	E.V.Zemlyanaya      	
c
c	HIGH-ENERGY APPROXIMATION
c
c	CALCULATION Of DIFFERENTIAL CROSS SECTION
c	(RELATIVIZATION EFFECT AND COLOUMB DISTURBTION ARE INCLUDED)
c	
c     WOODS-SAXON POTENTIAL IS APPLIED. 		
c	EXTERNAL REAL & IMAGINARY PHASE IS AVAILABLE.
c
c*********************************************************
c
      IMPLICIT real*8(A-H,p-Z)
      implicit complex*16 (o)
 
	parameter (mmm=5000)

      dimension a(mmm),b(mmm),c(mmm)
	dimension chi_im(mmm),chi_re(mmm)

      common /params_n/ rvb,av,rvbda,ri1
	common /params_nw/ rwb,aw,rwbda,ri1w

	character file1*20, file2*20, file3*20, file4*20	

 	real ieta

	external fff,fffw

      one   = (0.d0,1.d0)
      ozero = (0.d0,0.d0)

      pi    = 2.d0*asin(1.0d0)
      pi2   = pi*2.d0

	hc    = 197.327d0
      f938  = 931.495d0

      ff    = pi/180.d0
      etac  = 10.d0*ff


c	unit 11: real phase
c	unit 13: imaginary phase
c	unit 15: output file
c	unit 17: file of parameters
c
c !!!!!!	ATTENTION !!!!!! 
c !!!!!!  both phases should be positive !!!
c

	print *,'file of parameters:'
	read (*,*) file1
	open(17,file=file1, status='old')

c	print *,'output file:'
	read (17,*) file4
      open (15,file=file4,status='unknown')

c
c		Example: O16+Ca40; E_lab=1503
c
c----------------- input parameters to be changed
c
c	a1,z1: projective; a2,z2: target	      
c	tlab: E_lab in MeV

	read(17,*) tlab,a1,z1,a2,z2

c	 parameter of Coloumb potential
	
	read (17,*) rvc

c	nr: number of points; rmax: length of interval	
	     
	read (17,*) nr, rmax
	
c	output angles (degree)

	read (17,*) etamin, etamax, etastep

c	Read key1

c	key1=0: external phase; read data files.

	read (17,*) key1

	if(key1.eq.0) then 
	read (17,*) file2
      open (11,file=file2,status='old')
	read (17,*) file3
      open (13,file=file3,status='unknown')
	
	read (17,*) v_koef, w_koef 	
	else

c	nonzero key1: Woods-Saxon; read WS parameters.	
c     key2=0: numerical integration; nonzero key2: analytic approximation

	read (17,*) key2
	read(17,*) v0,w0,rv,av,rw,aw

      rvb   = rv*(a1**(1.d0/3.d0)+a2**(1.d0/3.d0))
      rvbig = rvb
      
      rvbda = rvb/av
      rda = rvbda

	rwb   = rw*(a1**(1.d0/3.d0)+a2**(1.d0/3.d0))
      rwbig = rwb
      
      rwbda = rwb/aw
      rwda = rwbda

	endif

c--------------------------------------
c--------------------------------------  CONSTANTS
c--------------------------------------

	e = tlab*a2/(a1+a2)

      hc2   = hc*hc
      cm    = sqrt(a1*a2/(a1+a2))

	am2 = a2*f938
	am1 = a1*f938

	ck = am2/hc*sqrt(tlab*(tlab+2.d0*am1))
     ^		/sqrt((am1+am2)**2+2.d0*am2*tlab)

      ck2   = ck*2.d0

	betaa= sqrt(tlab*(tlab+2.d0*am1))/(tlab+am1)

	hv1 = 1.d0/hc/betaa

	www = sqrt((ck*hc)**2+am2**2) + sqrt((ck*hc)**2+am1**2)
	gammab = ck/hv1*(www-am2)/((www-am2)**2-am1**2)

        rvbc = rvc*(a1**(1./3.)+a2**(1./3.))

        ekv = 1.44d0
        ezz = z1*z2*ekv*hv1
ccc

c        rms = sqrt(.6d0)*rvbc*sqrt(1.+7./3.*(avc*pi/rvbc)**2)
c        ru  = rms/sqrt(.6d0)

		ru = rvbc

        vb  = z1*z2*ekv/ru
        ac  = z1*z2*ekv/e/2.d0

      rmin  = 1.e-3
      nrm1  = nr-1
      hr    = (rmax-rmin)/nrm1

      do i  = 1,nr
      b(i)  = rmin+hr*(i-1)
      end do

c
c--------------------------------------------------
c------------------------------- EXTERNAL PHASE
c--------------------------------------------------
c
c
	if(key1.ne.0) goto 999			

	do i=1,nr
	read (13,*) xxx, chi_im(i)
	read (11,*) xxx, chi_re(i)
	end do
c
c------------------------------- START CYCLE ON THE ANGLE
c
      do ieta = etamin, etamax, etastep

      etar  = ieta*ff
      etar2 = etar/2.d0
      sinet = sin(etar2)
      coset = cos(etar2)
      q     = ck2*sinet
      fa  = 2.*ezz*dlog(ck*rmax)
ccc
      do i  = 1,nr
      ri    = b(i)
      q1    = q*coset
      q2    = q*sinet
      q1b   = ri*q
      ckb   = ck*ri
      ri1 = ac+sqrt(ac**2+ri**2)
c
c--------------------------------------- Bessel
c
       if(abs(q1b).le.3.d0) then
c
c--------------------------------------- Abramovic, p.191
c
       xx   = (q1b/3.d0)**2
       bj0  = 1.d0-2.2499997d0*xx+1.2656208d0*xx**2-.3163866d0*xx**3
     ^      + .0444479d0*xx**4-.0039444d0*xx**5+.00021d0*xx**6
       bj1  = (.5d0 - .56249985d0*xx + .21093573d0*xx**2
     ^      - .03954289d0*xx**3 + .00443319d0*xx**4
     ^      - .00031761d0*xx**5 + .00001109d0*xx**6)*q1b
      else
c--------------------------------------- Abramovic, p.191,eq.9.4.3
       xx   = (3.d0/q1b)
       f0   = .79788456d0-.00000077d0*xx-.00552740d0*xx**2
     ^      - .00009512d0*xx**3+.00137237d0*xx**4-.00072805d0*xx**5
     ^      + .00014476d0*xx**6
       t0   = q1b-.78539816d0-.04166397d0*xx-.00003954d0*xx**2
     ^      + .00262573d0*xx**3-.00054125d0*xx**4-.00029333d0*xx**5
     ^      + .00013558d0*xx**6
       bj0  = q1b**(-.5d0)*f0*cos(t0)
c
       f1   = .79788456d0+.00000156d0*xx+.01659667d0*xx**2
     ^      + .00017105d0*xx**3+.00249511d0*xx**4+.00113653d0*xx**5
     ^      - .00020033d0*xx**6
       t1   = q1b-2.35619446d0+.12499612d0*xx+.0000565d0*xx**2
     ^      - .00637879d0*xx**3+.000743484125d0*xx**4+.00079824d0*xx**5
     ^      + .00029166d0*xx**6

       bj1  = q1b**(-.5d0)*f1*cos(t1)

       end if
c
c------------------- F_{N}
c
	ofn   = cmplx( v_koef*chi_re(i),w_koef*chi_im(i))
c
c------------------- F_{uc}
c
        if(ri.gt.ru) then
        fuc_1  = 0.
        else
        fuc_1  = 2.d0*ezz * (log(ck*ru)-sqrt(ru**2-ri**2)/ru
     ^         - 1.d0/3.d0/ru**3*(ru**2-ri**2)**1.5d0
     ^         + log(1.d0+sqrt(ru**2-ri**2)/ru))
     ^         - 2.d0*ezz*log(ck*ri)
        end if
ccc
        fuc = fuc_1
c
c------------------- F_{N} + F_{uc}
c
       oarg  = ri*bj0*(1.d0-exp(one*(fuc+ofn)))*
     ^         exp(one*2.d0*ezz*log(ck*ri))
       c(i)  = real(oarg)
       a(i)  = imag(oarg)
c
        end do
c
      call euler(nr,hr,c,tjr)
      call euler(nr,hr,a,tji)
c
c---------------------------------- sigma_Ruth
c
      if(ezz.lt.1.d0) then
		sig0   = ezz*(1.d0-.5772d0)-atan(ezz)
      else
	    sig0   = ezz*log(ezz)-ezz+pi/4.d0
      endif

      ofpc=-ezz/q/sinet*exp(-one*ezz*2.d0*log(sinet)+2.d0*one*sig0)
      znam= (ezz/q/sinet)**2

c--------------------------------- our diff. sigma

c of1:
        of1  = ck*one*cmplx(tjr,tji)
        of1  = of1+ofpc
        oie  = 2.d0*one*ezz
        of2  = ck*one*(oie*bj0*(1.-((oie-1.d0)**2-1.d0)/q1b**2
     ^       + ((oie-1.d0)**2-1.d0)*(oie-3.d0)**2-1.d0)/q1b**4)
     ^       - bj1*(oie**2/q1b-oie**2*(oie-2)**2/q1b**3)
      oampl  = of1

c of2:
      smod   = real(oampl)**2+aimag(oampl)**2
      smod1  = real(of1)**2+aimag(of1)**2
      smod2  = real(of2)**2+aimag(of2)**2
c
      result1 = smod/znam
	etarf = etar/ff

c		write results:
c		etarf: angle (degree); 
c		result1: dsig/dsig_ruth; 
c		smod: diffferential cross section

      write (15,1000) etarf,result1,smod

ccc	print *,etarf,result1

      end do

	stop
c
c-----------------------------------
c-------------------	WOODS-SAXON
c-----------------------------------
c
 999  continue 

      oconst1 = -(v0+one*w0)*hv1*gammab
      oconst2 = -(v0+one*w0)*f938*a1*a2/(a1+a2)/hc2

c
c-------------------- start cycle on angle
c
      do ieta = etamin, etamax, etastep
      etar  = ieta*ff
      etar2 = etar/2.d0
      sinet = sin(etar2)
      coset = cos(etar2)
      q     = ck2*sinet
      fa  = 2.d0*ezz*dlog(ck*rmax)
ccc
      do i  = 1,nr
      ri    = b(i)
      q1    = q*coset
      q2    = q*sinet
      q1b   = ri*q
      ckb   = ck*ri
      ri1 = ac+sqrt(ac**2+ri**2)
	ri1w = ri1
c
       if(abs(q1b).le.3.d0) then
c--------------------------------------- Abramovic, p.191
       xx   = (q1b/3.d0)**2
       bj0  = 1.d0-2.2499997d0*xx+1.2656208d0*xx**2-.3163866d0*xx**3
     ^      + .0444479d0*xx**4-.0039444d0*xx**5+.00021d0*xx**6
       bj1  = (.5d0 - .56249985d0*xx + .21093573d0*xx**2
     ^      - .03954289d0*xx**3 + .00443319d0*xx**4
     ^      - .00031761d0*xx**5 + .00001109d0*xx**6)*q1b
      else
c--------------------------------------- Abramovic, p.191,eq.9.4.3
       xx   = (3.d0/q1b)
       f0   = .79788456d0-.00000077d0*xx-.00552740d0*xx**2
     ^      - .00009512d0*xx**3+.00137237d0*xx**4-.00072805d0*xx**5
     ^      + .00014476d0*xx**6
       t0   = q1b-.78539816d0-.04166397d0*xx-.00003954d0*xx**2
     ^      + .00262573d0*xx**3-.00054125d0*xx**4-.00029333d0*xx**5
     ^      + .00013558d0*xx**6
       bj0  = q1b**(-.5d0)*f0*cos(t0)
c
       f1   = .79788456d0+.00000156d0*xx+.01659667d0*xx**2
     ^      + .00017105d0*xx**3+.00249511d0*xx**4+.00113653d0*xx**5
     ^      - .00020033d0*xx**6
       t1   = q1b-2.35619446d0+.12499612d0*xx+.0000565d0*xx**2
     ^      - .00637879d0*xx**3+.000743484125d0*xx**4+.00079824d0*xx**5
     ^      + .00029166d0*xx**6

       bj1  = q1b**(-.5d0)*f1*cos(t1)

       end if
c
        bj2 = 2.d0/q1b*bj1-bj0
c
c------------------- F_{N}
c
	if(key2.eq.0) then

        call Gauss_Integration(rmin,rmax,fff,c_num)
	  call Gauss_Integration(rmin,rmax,fffw,c_numw)

	c_num = c_num*2.d0
	c_numw = c_numw*2.d0
      ofn   = real(oconst1)*c_num+imag(oconst1)*c_numw*one

	else
ccc
       cnn   = 10**(.47909d0+.15025d0*rda-.001938d0*rda**2)
	 cnnw   = 10**(.47909d0+.15025d0*rwda-.001938d0*rwda**2)

        beta = ri1/rvb
	  betaw = ri1w/rwb
ccc
      p1ac  = 2.4895453d0/rda+.34597d0-.0046*rda
	p1acw  = 2.4895453d0/rwda+.34597d0-.0046*rwda

      si1b  = sinh(rda)/(cosh(rda)+cosh(beta*rda))
	si1bw  = sinh(rwda)/(cosh(rwda)+cosh(betaw*rwda))

ccc
        cbeta = ri1/av
	  cbetaw = ri1w/aw

        apb = cosh(rda)+cosh(cbeta)
	  apbw = cosh(rwda)+cosh(cbetaw)

        amb = apb-(2.d0/cnn*(cosh(cbeta)-1.d0)+2.d0)
	  ambw = apbw-(2.d0/cnnw*(cosh(cbetaw)-1.d0)+2.d0)

        sss = 1.d0/sqrt(apb)/sqrt(amb)
	  sssw = 1.d0/sqrt(apbw)/sqrt(ambw)

        sss1 = log((sqrt(apb)+sqrt(amb))/(sqrt(apb)-sqrt(amb)))
	  sss1w = log((sqrt(apbw)+sqrt(ambw))/(sqrt(apbw)-sqrt(ambw)))

        sss = sss*sss1
	 sssw = sssw*sss1w

	  sss = sss*2.d0*av*sinh(rda)
	 sssw = sssw*2.d0*aw*sinh(rwda)

        ofn = real(oconst1)*sss + one*imag(oconst1)*sssw

	endif
ccc
        if(ri.gt.ru) then
        fuc_1  = 0.d0
        else
        fuc_1  = 2.d0*ezz * (dlog(ck*ru)-(ru**2-ri**2)**.5/ru
     ^         - 1.d0/3.d0/ru**3*(ru**2-ri**2)**1.5
     ^         + dlog(1.d0+(ru**2-ri**2)**.5/ru))
     ^         - 2.d0*ezz*dlog(ck*ri)
        end if
ccc
        fuc = fuc_1
c
c------------------- F_{N} + F_{uc}
c
       oarg  = ri*bj0*(1.d0-exp(one*(fuc+ofn)))*
     ^         exp(one*2.d0*ezz*dlog(ck*ri))
       c(i)  = real (oarg)
       a(i)  = imag(oarg)
c
        end do
c
      call euler(nr,hr,c,tjr)
      call euler(nr,hr,a,tji)
c
c---------------------------------- sigma_R
c
        if(ezz.lt.1.d0)then
      sig0   = ezz*(1.d0-.5772d0)-atan(ezz)

      else
      sig0   = ezz*log(ezz)-ezz+pi/4.d0
      endif
      ofpc= -ezz/q/sinet*exp(-one*ezz*2.d0*log(sinet)+2.d0*one*sig0)
      znam= (ezz/q/sinet)**2

c
c--------------------------------- our sigma
c
        of1  = ck*one*dcmplx(tjr,tji)
        of1  = of1+ofpc
        oie  = 2.d0*one*ezz
        of2  = ck*one*(oie*bj0*(1.d0-((oie-1.d0)**2-1.d0)/q1b**2
     ^       + ((oie-1)**2-1.d0)*(oie-3.d0)**2-1.d0)/q1b**4)
     ^       - bj1*(oie**2/q1b-oie**2*(oie-2.d0)**2/q1b**3)
      oampl  = of1
cof2
      smod   = real(oampl)**2+imag(oampl)**2
      smod1  = real(of1)**2+imag(of1)**2
      smod2  = real(of2)**2+imag(of2)**2
c
      result = smod/znam

	etarf = etar/ff


c		write results:
c		etarf: angle (degree); 
c		result1: dsig/dsig_ruth; 
c		smod: diffferential cross section

      write (15,1000) etarf,result,smod
c	print *,etarf,result
c
      end do

 1000 format(5e15.7)
      end
c*
c*****************************************************************
c*
      subroutine euler(n,h,f,s2)
      implicit real*8 (a-h,o-z)
      dimension f(n)
      s    = (f(1)+f(n))/2.
      nm1  = n-1
      do 1 i=2,nm1
 1    s    = s+f(i)
      ss   = s*h
      h2   = h*2.d0
      hh   = h*h
      df12 = (-3.d0*f(1)+4.d0*f(2)-f(3))/h2
      dfn2 = (f(n-2)-4.d0*f(nm1)+3.d0*f(n))/h2

      S2   = SS+hh*(DF12-DFN2)/12.d0

      return
      end

c*
c***********************************************************
c*
      subroutine Gauss_Integration(A,B,FF,outcome)
      implicit real*8 (a-h,o-z)
C*
      common /absolute_error/ error
      dimension X(12),W1(12),W2(12)
      DATA DELTA/1.D-10/,EK/25.D0/,AK/.5D0/, aeps/1.0d-7/

      DATA X/ 0.001815193055229D0, 0.010885670926972D0,
     *        0.029161445710966D0, 0.056468700115952D0,
     *        0.091971271671890D0, 0.134923997212975D0,
     *        0.184700239919018D0, 0.240451935396594D0,
     *        0.301027929523811D0, 0.365228422023828D0,
     *        0.431943499600319D0, 0.5D0 /
      DATA W1/ 0.0D0, 0.027834283558087D0, 0.0D0, 0.062790184732452D0,
     *         0.0D0, 0.093145105463867D0, 0.0D0, 0.116596882295995D0,
     *         0.0D0, 0.131402272255123D0, 0.0D0, 0.136462543388950D0/
      DATA W2/0.004882720522980D0, 0.013578277341052D0,
     *        0.022914689282213D0, 0.031548712375188D0,
     *        0.039332285966114D0, 0.046476549298450D0,
     *        0.052936037240695D0, 0.058369751230524D0,
     *        0.062579399550160D0, 0.065640342 114903D0,
     *        0.067596786399942D0, 0.068288897355559D0/
      gint=0.0d0
      error=0.0d0
      a1=a
      b1=b
      s=1.0d0
      if(a-b) 12,10,11
  11  a1=b
      b1=a
      s=-s
  12  EPS=EK*AEPS
      ABSINT=0.D0
      I=1
    1 GINT=0.D0
      ERROR=0.D0
      AI=ABSINT
      AA=A1
    2 Y=B1-AA
      IF (DELTA.GT.ABS(Y)) GOTO 9
    3 BB=AA+Y
      CC=BB-AA
       aaaa = AA+CC*X(12)
       call FF(aaaa,fl1)
        ext=fl1
      S1=EXT*W1(12)
      S2=EXT*W2(12)
      do N=1,11
        DD=CC*X(N)
        call FF(AA+DD,fl1)
        call FF(BB-DD,fl2)
        EXT=fl1+fl2
        S1=S1+W1(N)*EXT
        S2=S2+W2(N)*EXT
      enddo
      S1=CC*S1
      S2=CC*S2
      BI=abs(S2)
      ER=abs(S1-S2)
      IF (I.GT.1) GOTO 5
      AI=abs(S2)
      I=2
    5 IF (ER/(1.D0+AI).LE.EPS.OR.ER/(1.D0+BI).LE.EPS) GOTO 7
    6 IF (DELTA.GE.abs(Y))  GOTO 8
      Y=.5D0*Y
      GOTO 3
    7 IF (ER/EPS.GT.AI.AND.ER/EPS.GT.BI) GOTO 6
    8 GINT=GINT+S2
      ERROR=ERROR+ER
      AA=BB
      GOTO 2
    9 ABSINT=abs(GINT)
      ERROR=ERROR/EK
      IF (ERROR/(1.D0+ABSINT).GT.AEPS.AND.ABSINT.LT.AK*AI) GOTO 1
      IF (ERROR/AEPS.GT.ABSINT.AND.ABSINT.LT.AK*AI)        GOTO 1
   10   continue

      outcome=gint*s

      RETURN
      END
c*
c***********************************************************
c*
        subroutine fff(x,a)
        implicit real*8 (a-h,o-z)
        common /params_n/ rvb,av,rvbda,ri1

        arg   = sqrt(x**2+ri1**2)
        a  = sinh(rvbda)/(cosh(rvbda)+cosh(arg/av))
c
        return
        end
c*
c*
c***********************************************************
c*
        subroutine fffw(x,a_imag)
        implicit real*8 (a-h,o-z)
        common /params_nw/ rwb,aw,rwbda,ri1w

        arg   = sqrt(x**2+ri1w**2)
        a_imag  = sinh(rwbda)/(cosh(rwbda)+cosh(arg/aw))
c
        return
        end
c*
